<?php

declare(strict_types=1);

namespace Drupal\Tests\menu_migration\Functional;

use Drupal\user\UserInterface;
use Drupal\menu_migration\Form\QuickActionSettingsForm;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\menu_migration\Traits\TestsHelperTrait;

/**
 * Tests the UI for the Menu Migration module.
 *
 * @group menu_migration
 */
class MenuMigrationUiTest extends BrowserTestBase {
  use TestsHelperTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['block', 'menu_migration'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A test user that has full Menu Migration admin permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->placeDefaultBlocks();
    // Create test users (the first one will have user ID 1 and won't be used).
    $this->drupalCreateUser();
    $this->adminUser = $this->drupalCreateUser(['administer menu migration']);
  }

  /**
   * Tests the module overview page.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testOverviewPage(): void {
    // Test the Overview page.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/menu-migration');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementTextEquals('xpath', '//h1', 'Menu Migration');
    $this->assertSession()->elementsCount('css', '.list-group__link', 3);
    $this->assertSession()->linkExistsExact('Menu Exports');
    $this->assertSession()->linkByHrefExists('/admin/config/development/menu-migration/export-types');
    $this->assertSession()->linkExistsExact('Menu Imports');
    $this->assertSession()->linkByHrefExists('/admin/config/development/menu-migration/import-types');
    $this->assertSession()->linkExistsExact('Quick Action Settings');
    $this->assertSession()->linkByHrefExists('/admin/config/development/menu-migration/quick-action-settings');
  }

  /**
   * Tests the UI of the empty Export Type and Import Type listing pages.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Behat\Mink\Exception\ResponseTextException
   */
  public function testEmptyListingPages(): void {
    // Test the Menu Exports listing page.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/menu-migration/export-types');
    $this->assertPrimaryTabs();
    $this->assertSession()->elementTextEquals('xpath', '//h1', 'Menu exports');
    $this->assertSession()->elementTextEquals('css', '.button-action', 'Add menu export');
    $tableHeaders = [
      '//*[@id="edit-export-types"]/thead/tr/th[1]' => 'ID',
      '//*[@id="edit-export-types"]/thead/tr/th[2]' => 'Name',
      '//*[@id="edit-export-types"]/thead/tr/th[3]' => 'Menus',
      '//*[@id="edit-export-types"]/thead/tr/th[4]' => 'Format',
      '//*[@id="edit-export-types"]/thead/tr/th[5]' => 'Destination',
      '//*[@id="edit-export-types"]/thead/tr/th[6]' => 'Drush command',
      '//*[@id="edit-export-types"]/thead/tr/th[7]' => 'Weight',
      '//*[@id="edit-export-types"]/thead/tr/th[8]' => 'Operations',
    ];
    foreach ($tableHeaders as $selector => $header) {
      $this->assertSession()->elementTextEquals('xpath', $selector, $header);
    }
    $this->assertSession()->pageTextContains('There are no menu exports yet.');
    $this->assertSession()->elementExists('xpath', '//input[@id="edit-submit" and @value="Save"]');

    $this->clickLink('Add menu export');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementTextEquals('xpath', '//h1', 'Add menu export');

    // Test the Menu Imports listing page.
    $this->drupalGet('/admin/config/development/menu-migration/import-types');
    $this->assertPrimaryTabs();
    $this->assertSession()->elementTextEquals('xpath', '//h1', 'Menu imports');
    $this->assertSession()->elementTextEquals('css', '.button-action', 'Add menu import');
    $tableHeaders = [
      '//*[@id="edit-import-types"]/thead/tr/th[1]' => 'ID',
      '//*[@id="edit-import-types"]/thead/tr/th[2]' => 'Name',
      '//*[@id="edit-import-types"]/thead/tr/th[3]' => 'Menus',
      '//*[@id="edit-import-types"]/thead/tr/th[4]' => 'Format',
      '//*[@id="edit-import-types"]/thead/tr/th[5]' => 'Source',
      '//*[@id="edit-import-types"]/thead/tr/th[6]' => 'Drush command',
      '//*[@id="edit-import-types"]/thead/tr/th[7]' => 'Weight',
      '//*[@id="edit-import-types"]/thead/tr/th[8]' => 'Operations',
    ];
    foreach ($tableHeaders as $selector => $header) {
      $this->assertSession()->elementTextEquals('xpath', $selector, $header);
    }
    $this->assertSession()->pageTextContains('There are no menu imports yet.');
    $this->assertSession()->elementExists('xpath', '//input[@id="edit-submit" and @value="Save"]');

    $this->clickLink('Add menu import');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementTextEquals('xpath', '//h1', 'Add menu import');
  }

  /**
   * Tests the view, update, delete and export of a Codebase Export Type.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testCodebaseExportTypeActions(): void {
    $this->generateMenuItems();
    $exportFolder = '../config/menu_migration/' . $this->randomMachineName();
    $this->exportTypeStorage()->create([
      'label' => 'Some Codebase Export',
      'name' => 'some_codebase_export',
      'destination' => 'codebase',
      'destination_config' => [
        'format' => 'yaml',
        'menus' => [
          'footer' => 'footer',
          'main' => 'main',
        ],
        'export_path' => $exportFolder,
      ],
    ])->save();
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/menu-migration/export-types');
    $this->assertSession()->pageTextNotContains('There are no menu exports yet.');
    $this->assertEntityTableColumns([
      'some_codebase_export',
      'Some Codebase Export',
      'footer, main',
      'YAML',
      "Codebase$exportFolder",
      'drush mme some_codebase_export',
    ], 'export');

    // Test export confirmation form.
    $this->clickLink('Export');
    $assertSession = $this->assertSession();
    $assertSession->pageTextContains('Are you sure you want to export Some Codebase Export?');
    $assertSession->pageTextContains('The following menus: footer, main will be exported to the Codebase destination using the YAML format.');
    $assertSession->pageTextContains('Your menus will be exported to:');
    $assertSession->pageTextContains("$exportFolder/footer.yaml");
    $assertSession->pageTextContains("$exportFolder/main.yaml");

    $assertSession->linkExistsExact('Cancel');
    $assertSession->linkByHrefExists('/admin/config/development/menu-migration/export-types');

    $this->getSession()->getPage()->pressButton('Export');
    $this->assertSession()->statusMessageContains('Successfully exported the following menu(s): footer, main');
    $this->assertFileExists(DRUPAL_ROOT . "/$exportFolder/footer.yaml");
    $this->assertFileExists(DRUPAL_ROOT . "/$exportFolder/main.yaml");

    // Edit the Export Type and change the format, then delete it.
    $this->assertEditFormat('export', 'destination_config');
    $this->assertDeleteEntity('export', 'some_codebase_export');
  }

  /**
   * Tests the view, update, delete and import of a Codebase Import Type.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testCodebaseImportTypeActions(): void {
    $modulePath = $this->container->get('extension.list.module')->getPath('menu_migration');
    $importFolder = $modulePath . '/tests/fixtures';
    $this->importTypeStorage()->create([
      'label' => 'Main and Footer Menus',
      'name' => 'main_and_footer_import',
      'source' => 'codebase',
      'source_config' => [
        'format' => 'yaml',
        'menus' => [
          'footer' => 'footer',
          'main' => 'main',
        ],
        'import_path' => $importFolder,
      ],
    ])->save();
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/menu-migration/import-types');
    $this->assertSession()->pageTextNotContains('There are no menu imports yet.');
    $this->assertEntityTableColumns([
      'main_and_footer_import',
      'Main and Footer Menus',
      'footer, main',
      'YAML',
      "Codebase$importFolder",
      'drush mmi main_and_footer_import',
    ], 'import');

    // Test export confirmation form.
    $this->clickLink('Import');
    $assertSession = $this->assertSession();
    $assertSession->pageTextContains('Are you sure you want to import Main and Footer Menus?');
    $assertSession->pageTextContains('The following menus: footer, main will be imported from the Codebase source using the YAML format.');
    $assertSession->pageTextContains('All existing menu items will be deleted prior to the import.');
    $assertSession->pageTextContains('Your menus will be imported from:');
    $assertSession->pageTextContains("$importFolder/footer.yaml");
    $assertSession->pageTextContains("$importFolder/main.yaml");

    $assertSession->linkExistsExact('Cancel');
    $assertSession->linkByHrefExists('/admin/config/development/menu-migration/import-types');

    $this->getSession()->getPage()->pressButton('Import');
    $this->assertSession()->statusMessageContains('Successfully imported the following menu(s): footer, main');

    // Edit the Export Type and change the format, then delete it.
    $this->assertEditFormat('import', 'source_config');
    $this->assertDeleteEntity('import', 'main_and_footer_import');
  }

  /**
   * Tests the view, update, delete and export of a Download Export Type.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testDownloadExportTypeActions(): void {
    $this->generateMenuItems();
    $this->exportTypeStorage()->create([
      'label' => 'A Download Export',
      'name' => 'download_export',
      'destination' => 'download',
      'destination_config' => [
        'format' => 'yaml',
        'menus' => 'main',
      ],
    ])->save();
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/menu-migration/export-types');
    $this->assertSession()->pageTextNotContains('There are no menu exports yet.');
    $this->assertEntityTableColumns([
      'download_export',
      'A Download Export',
      'main',
      'YAML',
      'Download',
      'N/A',
    ], 'export');

    // Test export confirmation form.
    $this->clickLink('Export');
    $assertSession = $this->assertSession();
    $assertSession->pageTextContains('Are you sure you want to export A Download Export?');
    $assertSession->pageTextContains('The following menu: main will be exported to the Download destination using the YAML format.');
    $assertSession->pageTextContains('The selected menu will be exported as a downloadable file.');
    $assertSession->pageTextContains('Click here after you finish downloading the file.');
    $assertSession->linkExistsExact('here');
    $assertSession->linkByHrefExists('/admin/config/development/menu-migration/export-types');

    $assertSession->linkExistsExact('Cancel');
    $assertSession->linkByHrefExists('/admin/config/development/menu-migration/export-types', 1);

    // Edit the Export Type and change the format, export it again with the new
    // format and then delete the entity.
    $this->assertExportDownloadHeaders('attachment; filename="main.yaml"', 'text/plain;charset=UTF-8');
    $this->assertEditFormat('export', 'destination_config');
    $this->clickLink('Export');
    $this->assertExportDownloadHeaders('attachment; filename="main.json"', 'application/json');
    $this->drupalGet('/admin/config/development/menu-migration/export-types');
    $this->assertDeleteEntity('export', 'download_export');
  }

  /**
   * Tests the view, update, delete and import of a File Upload Import Type.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testFileUploadImportTypeActions(): void {
    $modulePath = $this->container->get('extension.list.module')->getPath('menu_migration');
    $importFolder = $modulePath . '/tests/fixtures';
    $this->importTypeStorage()->create([
      'label' => 'Footer import',
      'name' => 'footer_import',
      'source' => 'file_upload',
      'source_config' => [
        'format' => 'yaml',
        'menus' => 'footer',
      ],
    ])->save();
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/menu-migration/import-types');
    $this->assertSession()->pageTextNotContains('There are no menu imports yet.');
    $this->assertEntityTableColumns([
      'footer_import',
      'Footer import',
      'footer',
      'YAML',
      'File Upload',
      'N/A',
    ], 'import');

    // Test export confirmation form.
    $this->clickLink('Import');
    $assertSession = $this->assertSession();
    $assertSession->pageTextContains('Are you sure you want to import Footer import?');
    $assertSession->pageTextContains('The following menu: footer will be imported from the File Upload source using the YAML format.');
    $assertSession->pageTextContains('All existing menu items will be deleted prior to the import.');
    $assertSession->pageTextContains('The selected menu will be imported from the uploaded file.');
    $assertSession->pageTextContains('Upload a file containing the menu items to import. Allowed extensions: yaml, yml.');
    $this->getSession()->getPage()->attachFileToField('files[action_form]', $this->root . '/' . $importFolder . '/footer.yaml');

    $assertSession->linkExistsExact('Cancel');
    $assertSession->linkByHrefExists('/admin/config/development/menu-migration/import-types');

    $this->getSession()->getPage()->pressButton('Import');
    $this->assertSession()->statusMessageContains('Successfully imported the following menu(s): footer');

    // Edit the Export Type and change the format, then delete it.
    $this->assertEditFormat('import', 'source_config');
    $this->clickLink('Import');
    $assertSession->pageTextContains('Upload a file containing the menu items to import. Allowed extensions: json.');
    $this->drupalGet('/admin/config/development/menu-migration/import-types');
    $this->assertDeleteEntity('import', 'footer_import');
  }

  /**
   * Tests the Quick Action Settings form.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Behat\Mink\Exception\ResponseTextException
   */
  public function testQuickActionsForm(): void {
    $this->drupalLogin($this->adminUser);
    // Test the Quick Action Settings page.
    $this->drupalGet('/admin/config/development/menu-migration/quick-action-settings');
    $this->assertPrimaryTabs();
    $this->assertSession()->elementTextEquals('xpath', '//h1', 'Quick Action Settings');
    // Ensure default values for quick export settings.
    $settings = $this->config(QuickActionSettingsForm::QUICK_EXPORT_SETTINGS);
    $this->assertEquals('json', $settings->get('format'));
    $this->assertEquals('../config/menu_migration/quick-export', $settings->get('export_path'));

    // Verify fields presence.
    $this->assertSession()->selectExists('format');
    $this->assertSession()->fieldExists('export_path');

    // Verify changes.
    $edit = ['format' => 'yaml', 'export_path' => '../assets/menus'];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->pageTextContains('The configuration options have been saved.');
    $updatedConfig = $this->config(QuickActionSettingsForm::QUICK_EXPORT_SETTINGS);
    $this->assertEquals('yaml', $updatedConfig->get('format'));
    $this->assertEquals('../assets/menus', $updatedConfig->get('export_path'));
  }

  /**
   * Verifies the existence of the Menu Migration primary tabs.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  protected function assertPrimaryTabs(): void {
    $this->assertSession()->elementTextContains('xpath', '//ul/li[1]/a', 'Menu exports');
    $this->assertSession()->elementTextContains('xpath', '//ul/li[2]/a', 'Menu imports');
    $this->assertSession()->elementTextContains('xpath', '//ul/li[3]/a', 'Quick Action Settings');
    $this->assertSession()->linkByHrefExists('/admin/config/development/menu-migration/export-types');
    $this->assertSession()->linkByHrefExists('/admin/config/development/menu-migration/import-types');
    $this->assertSession()->linkByHrefExists('/admin/config/development/menu-migration/quick-action-settings');
  }

  /**
   * Edits the format of the current entity and verifies the change.
   *
   * @param string $type
   *   The type of entity: "export" or "import".
   * @param string $formatParent
   *   The parent ID of the format element in the form. E.g. "source_config" or
   *   "destination_config".
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  protected function assertEditFormat(string $type, string $formatParent): void {
    $this->drupalGet("/admin/config/development/menu-migration/$type-types");
    $this->clickLink('Edit');
    $this->getSession()->getPage()->selectFieldOption("{$formatParent}[format]", 'json');
    $this->getSession()->getPage()->pressButton('Save');
    $this->assertSession()->addressEquals("/admin/config/development/menu-migration/$type-types");
    $this->assertSession()->elementTextEquals('xpath', '//*[@id="edit-' . $type . '-types"]/tbody/tr/td[4]', 'JSON');
  }

  /**
   * Checks the deletion of an entity in the UI.
   *
   * @param string $type
   *   The type of entity: "export" or "import".
   * @param string $entityId
   *   The name property of the entity (which is also the ID).
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Behat\Mink\Exception\ResponseTextException
   */
  protected function assertDeleteEntity(string $type, string $entityId): void {
    $this->clickLink('Delete');
    $this->assertSession()->addressEquals("/admin/config/development/menu-migration/$type-types/$entityId/delete");
    $this->getSession()->getPage()->pressButton('Delete');
    $this->assertSession()->addressEquals("/admin/config/development/menu-migration/$type-types");
    $this->assertSession()->pageTextContains("There are no menu {$type}s yet.");
  }

  /**
   * Verifies the request headers for a triggered export download.
   *
   * @param string $expectedContentDisposition
   *   The expected value of the "Content-Disposition" header.
   * @param string $expectedContentType
   *   The expected value of the "Content-Type" header.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   */
  protected function assertExportDownloadHeaders(string $expectedContentDisposition, string $expectedContentType): void {
    $this->getSession()->getPage()->pressButton('Export');
    // Check the response headers for the downloaded file.
    $session = $this->getSession();
    $headers = $session->getResponseHeaders();
    $this->assertEquals($expectedContentDisposition, $headers['Content-Disposition'][0]);
    $this->assertEquals($expectedContentType, $headers['Content-Type'][0]);
  }

  /**
   * Generates some menu items in the "main" and "footer" menus.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function generateMenuItems(): void {
    // Generate some random menu items.
    for ($i = 1; $i <= 3; $i++) {
      $this->menuLinkContentStorage()->create([
        'title' => $this->randomString(),
        'menu_name' => 'main',
        'link' => ['uri' => "internal:/link$i"],
        'weight' => $i,
      ])->save();
      $this->menuLinkContentStorage()->create([
        'title' => $this->randomString(),
        'menu_name' => 'footer',
        'link' => ['uri' => "internal:/link$i"],
        'weight' => $i,
      ])->save();
    }
  }

}
