<?php

declare(strict_types=1);

namespace Drupal\Tests\menu_migration\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\menu_migration\Traits\TestsHelperTrait;
use Drupal\user\UserInterface;

/**
 * Tests the UI pages access by permissions.
 *
 * @group menu_migration
 */
class PermissionsUiTest extends BrowserTestBase {
  use TestsHelperTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['block', 'menu_migration'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A test user that has full Menu Migration admin permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $bypassUser;

  /**
   * A test user that can only administer export types.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $exportAdmin;

  /**
   * A test user that can only perform exports.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $exportPerformer;

  /**
   * A test user that can only administer import types.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $importAdmin;

  /**
   * A test user that can only perform imports.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $importPerformer;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->placeDefaultBlocks();
    // Create test users (the first one will have user ID 1 and won't be used).
    $this->drupalCreateUser();
    $this->bypassUser = $this->drupalCreateUser(['administer menu migration']);
    $this->exportAdmin = $this->drupalCreateUser(['administer menu migration export types']);
    $this->exportPerformer = $this->drupalCreateUser(['perform export on menu migrations']);
    $this->importAdmin = $this->drupalCreateUser(['administer menu migration import types']);
    $this->importPerformer = $this->drupalCreateUser(['perform import on menu migrations']);

    $this->exportTypeStorage()->create([
      'label' => 'Codebase Export',
      'name' => 'codebase_export',
      'destination' => 'codebase',
      'destination_config' => [
        'format' => 'json',
        'menus' => ['main' => 'main', 'footer' => 'footer'],
      ],
      'weight' => 0,
    ])->save();
    $this->exportTypeStorage()->create([
      'label' => 'Download Export',
      'name' => 'download_export',
      'destination' => 'download',
      'destination_config' => [
        'format' => 'yaml',
        'menus' => 'main',
      ],
      'weight' => 1,
    ])->save();

    $this->importTypeStorage()->create([
      'label' => 'Codebase Import',
      'name' => 'codebase_import',
      'source' => 'codebase',
      'source_config' => [
        'format' => 'json',
        'menus' => ['main' => 'main', 'footer' => 'footer'],
      ],
      'weight' => 0,
    ])->save();
    $this->importTypeStorage()->create([
      'label' => 'File Upload Import',
      'name' => 'file_upload_import',
      'source' => 'file_upload',
      'source_config' => [
        'format' => 'yaml',
        'menus' => 'main',
      ],
      'weight' => 1,
    ])->save();
  }

  /**
   * Tests the access of the menu_migration overview admin page.
   */
  public function testOverviewPageAccess(): void {
    $this->assertOverviewPageAccess($this->bypassUser, 200, 3);
    $this->assertOverviewPageAccess($this->exportAdmin, 403, 0);
    $this->assertOverviewPageAccess($this->exportPerformer, 403, 0);
    $this->assertOverviewPageAccess($this->importAdmin, 403, 0);
    $this->assertOverviewPageAccess($this->importPerformer, 403, 0);
  }

  /**
   * Tests the access of the Export Types listing page and its links.
   */
  public function testExportListPageAccess(): void {
    $this->assertExportListPageAccess($this->bypassUser, 200, TRUE, TRUE, TRUE, TRUE);
    $this->assertExportListPageAccess($this->exportAdmin, 200, TRUE, TRUE, TRUE, TRUE);
    $this->assertExportListPageAccess($this->exportPerformer, 200, TRUE);
    $this->assertExportListPageAccess($this->importAdmin, 403);
    $this->assertExportListPageAccess($this->importPerformer, 403);
  }

  /**
   * Tests the access of the Import Types listing page and its links.
   */
  public function testImportListPageAccess(): void {
    $this->assertImportListPageAccess($this->bypassUser, 200, TRUE, TRUE, TRUE, TRUE);
    $this->assertImportListPageAccess($this->exportAdmin, 403);
    $this->assertImportListPageAccess($this->exportPerformer, 403);
    $this->assertImportListPageAccess($this->importAdmin, 200, TRUE, TRUE, TRUE, TRUE);
    $this->assertImportListPageAccess($this->importPerformer, 200, TRUE);
  }

  /**
   * Tests the access of the Quick Action Settings form page.
   */
  public function testQuickActionSettingsPageAccess(): void {
    $this->assertQuickActionSettingsPageAccess($this->bypassUser, 200);
    $this->assertQuickActionSettingsPageAccess($this->exportAdmin, 403);
    $this->assertQuickActionSettingsPageAccess($this->exportPerformer, 403);
    $this->assertQuickActionSettingsPageAccess($this->importAdmin, 403);
    $this->assertQuickActionSettingsPageAccess($this->importPerformer, 403);
  }

  /**
   * Checks the access of the menu_migration admin overview page.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user to log in with.
   * @param int $statusCode
   *   The expected page request status code.
   * @param int $visibleLinksCount
   *   The expected visible administration links on this page.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  protected function assertOverviewPageAccess(UserInterface $user, int $statusCode, int $visibleLinksCount): void {
    $this->drupalLogin($user);
    $this->assertPageAccess('/admin/config/development/menu-migration', $statusCode);
    $this->assertSession()->elementsCount('css', '.list-group__link', $visibleLinksCount);
    $this->drupalLogout();
  }

  /**
   * Checks the access of the Export Types listing page and its links.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user to log in with.
   * @param int $statusCode
   *   The expected page request status code.
   * @param bool $exportAccess
   *   Boolean indicating if the user has access to export entities.
   * @param bool $addAccess
   *   Boolean indicating if the user has access to add entities.
   * @param bool $editAccess
   *   Boolean indicating if the user has access to edit entities.
   * @param bool $deleteAccess
   *   Boolean indicating if the user has access to delete entities.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  protected function assertExportListPageAccess(UserInterface $user, int $statusCode, bool $exportAccess = FALSE, bool $addAccess = FALSE, bool $editAccess = FALSE, bool $deleteAccess = FALSE): void {
    $this->drupalLogin($user);
    $pagePath = '/admin/config/development/menu-migration/export-types';
    $this->assertPageAccess($pagePath, $statusCode);
    if ($statusCode == 200) {
      $links = [
        'Add menu export' => $addAccess,
        'Edit' => $editAccess,
        'Delete' => $deleteAccess,
        'Export' => $exportAccess,
      ];
      foreach ($links as $linkTitle => $linkAccess) {
        if ($linkAccess) {
          $this->assertSession()->linkExistsExact($linkTitle);
          $this->clickLink($linkTitle);
          $this->assertSession()->statusCodeEquals(200);
          $this->drupalGet($pagePath);
        }
        else {
          $this->assertSession()->linkNotExistsExact($linkTitle);
        }
      }
    }
    $this->drupalLogout();
  }

  /**
   * Checks the access of the Import Types listing page and its links.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user to log in with.
   * @param int $statusCode
   *   The expected page request status code.
   * @param bool $importAccess
   *   Boolean indicating if the user has access to import entities.
   * @param bool $addAccess
   *   Boolean indicating if the user has access to add entities.
   * @param bool $editAccess
   *   Boolean indicating if the user has access to edit entities.
   * @param bool $deleteAccess
   *   Boolean indicating if the user has access to delete entities.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  protected function assertImportListPageAccess(UserInterface $user, int $statusCode, bool $importAccess = FALSE, bool $addAccess = FALSE, bool $editAccess = FALSE, bool $deleteAccess = FALSE): void {
    $this->drupalLogin($user);
    $pagePath = '/admin/config/development/menu-migration/import-types';
    $this->assertPageAccess($pagePath, $statusCode);
    if ($statusCode == 200) {
      $links = [
        'Add menu import' => $addAccess,
        'Edit' => $editAccess,
        'Delete' => $deleteAccess,
        'Import' => $importAccess,
      ];
      foreach ($links as $linkTitle => $linkAccess) {
        if ($linkAccess) {
          $this->assertSession()->linkExistsExact($linkTitle);
          $this->clickLink($linkTitle);
          $this->assertSession()->statusCodeEquals(200);
          $this->drupalGet($pagePath);
        }
        else {
          $this->assertSession()->linkNotExistsExact($linkTitle);
        }
      }
    }
    $this->drupalLogout();
  }

  /**
   * Checks the Quick Access Settings admin page access.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user to log in with.
   * @param int $statusCode
   *   The expected page request status code.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  protected function assertQuickActionSettingsPageAccess(UserInterface $user, int $statusCode): void {
    $this->drupalLogin($user);
    $this->assertPageAccess('/admin/config/development/menu-migration/quick-action-settings', $statusCode);
    $this->drupalLogout();
  }

  /**
   * Checks the access of the given page and its expected status code.
   *
   * @param string $path
   *   The Drupal path to check against.
   * @param int $statusCode
   *   The expected page request status code.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  protected function assertPageAccess(string $path, int $statusCode): void {
    $this->drupalGet($path);
    $this->assertSession()->statusCodeEquals($statusCode);
  }

}
