<?php

declare(strict_types=1);

namespace Drupal\Tests\menu_migration\FunctionalJavascript;

use Drupal\user\UserInterface;
use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\Tests\menu_migration\Traits\TestsHelperTrait;

/**
 * Tests the entities add forms by source/destination plugin type.
 *
 * @group menu_migration
 */
class AjaxFormsUiTest extends WebDriverTestBase {
  use TestsHelperTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['menu_migration'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * The admin user that we are using for tests.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    // Create an initial user that will have the UID 1.
    $this->drupalCreateUser();
    $this->adminUser = $this->drupalCreateUser(['administer menu migration']);
  }

  /**
   * Tests the Export Type add form using the Codebase destination plugin.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   */
  public function testAddCodebaseExportType(): void {
    $session = $this->getSession();
    $page = $session->getPage();
    $assertSession = $this->assertSession();
    $this->drupalLogin($this->adminUser);
    $folderName = $this->randomMachineName(6);

    // Add codebase export type.
    $this->drupalGet('/admin/config/development/menu-migration/export-types/add');
    $this->assertSession()->elementExists('xpath', '//input[@id="edit-submit" and @value="Create new menu export"]');
    $page->fillField('Menu export name', 'Multiple menus to codebase');
    $this->assertSession()->waitForElementVisible('css', '#edit-label-machine-name-suffix .link');
    $page->pressButton('Edit');
    $page->fillField('name', 'export_multiple_codebase');
    $page->selectFieldOption('edit-destination', 'Codebase');
    $assertSession->assertWaitOnAjaxRequest();
    $page->selectFieldOption('destination_config[format]', 'json');
    // The Codebase plugin allows for multiple menu selection.
    $assertSession->elementExists('css', '[data-drupal-selector="edit-destination-config-menus"] .form-checkboxes');
    $page->checkField('Main navigation');
    $page->checkField('Footer');
    $page->fillField('destination_config[export_path]', "../assets/$folderName");
    $this->submitForm([], 'Create new menu export', 'mm-export-type-add-form');

    $this->drupalGet('/admin/config/development/menu-migration/export-types');
    // Verify that the newly created export type is displayed as expected in the
    // listing.
    $this->assertEntityTableColumns([
      'export_multiple_codebase',
      'Multiple menus to codebase',
      'footer, main',
      'JSON',
      "Codebase ../assets/$folderName",
      'drush mme export_multiple_codebase',
    ], 'export');
  }

  /**
   * Tests the Export Type add form using the Download destination plugin.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   */
  public function testAddDownloadExportType(): void {
    $session = $this->getSession();
    $page = $session->getPage();
    $assertSession = $this->assertSession();
    $this->drupalLogin($this->adminUser);

    // Add Download export type.
    $this->drupalGet('/admin/config/development/menu-migration/export-types/add');
    $this->assertSession()->elementExists('xpath', '//input[@id="edit-submit" and @value="Create new menu export"]');
    $page->fillField('Menu export name', 'Main navigation download');
    $this->assertSession()->waitForElementVisible('css', '#edit-label-machine-name-suffix .link');
    $page->pressButton('Edit');
    $page->fillField('name', 'download_main_nav');
    $page->selectFieldOption('edit-destination', 'Download');
    $assertSession->assertWaitOnAjaxRequest();
    $page->selectFieldOption('destination_config[format]', 'yaml');
    // The Download plugin allows for single menu selection only.
    $assertSession->elementExists('css', '[data-drupal-selector="edit-destination-config-menus"] .js-form-type-radio');
    $page->selectFieldOption('destination_config[menus]', 'main');
    $this->submitForm([], 'Create new menu export', 'mm-export-type-add-form');

    $this->drupalGet('/admin/config/development/menu-migration/export-types');
    // Verify that the newly created export type is displayed as expected in the
    // listing.
    $this->assertEntityTableColumns([
      'download_main_nav',
      'Main navigation download',
      'main',
      'YAML',
      'Download',
      'N/A',
    ], 'export');
  }

  /**
   * Tests the Import Type add form using the Codebase source plugin.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   */
  public function testAddCodebaseImportType(): void {
    $session = $this->getSession();
    $page = $session->getPage();
    $assertSession = $this->assertSession();
    $this->drupalLogin($this->adminUser);
    $folderName = $this->randomMachineName(6);

    // Add codebase import type.
    $this->drupalGet('/admin/config/development/menu-migration/import-types/add');
    $this->assertSession()->elementExists('xpath', '//input[@id="edit-submit" and @value="Create new menu import"]');
    $page->fillField('Menu import name', 'Multiple menus from codebase');
    $this->assertSession()->waitForElementVisible('css', '#edit-label-machine-name-suffix .link');
    $page->pressButton('Edit');
    $page->fillField('name', 'import_multiple_codebase');
    $page->selectFieldOption('edit-source', 'Codebase');
    $assertSession->assertWaitOnAjaxRequest();
    $page->selectFieldOption('source_config[format]', 'json');
    // The Codebase plugin allows for multiple menu selection.
    $assertSession->elementExists('css', '[data-drupal-selector="edit-source-config-menus"] .form-checkboxes');
    $page->checkField('Main navigation');
    $page->checkField('Footer');
    $page->fillField('source_config[import_path]', "../assets/$folderName");
    $this->submitForm([], 'Create new menu import', 'mm-import-type-add-form');

    $this->drupalGet('/admin/config/development/menu-migration/import-types');
    // Verify that the newly created import type is displayed as expected in the
    // listing.
    $this->assertEntityTableColumns([
      'import_multiple_codebase',
      'Multiple menus from codebase',
      'footer, main',
      'JSON',
      "Codebase ../assets/$folderName",
      'drush mmi import_multiple_codebase',
    ], 'import');
  }

  /**
   * Tests the Import Type add form using the File Upload source plugin.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   */
  public function testAddFileUploadImportType(): void {
    $session = $this->getSession();
    $page = $session->getPage();
    $assertSession = $this->assertSession();
    $this->drupalLogin($this->adminUser);

    // Add Download import type.
    $this->drupalGet('/admin/config/development/menu-migration/import-types/add');
    $this->assertSession()->elementExists('xpath', '//input[@id="edit-submit" and @value="Create new menu import"]');
    $page->fillField('Menu import name', 'Main navigation upload');
    $this->assertSession()->waitForElementVisible('css', '#edit-label-machine-name-suffix .link');
    $page->pressButton('Edit');
    $page->fillField('name', 'upload_main_nav');
    $page->selectFieldOption('edit-source', 'File Upload');
    $assertSession->assertWaitOnAjaxRequest();
    $page->selectFieldOption('source_config[format]', 'yaml');
    // The Download plugin allows for single menu selection only.
    $assertSession->elementExists('css', '[data-drupal-selector="edit-source-config-menus"] .js-form-type-radio');
    $page->selectFieldOption('source_config[menus]', 'main');
    $this->submitForm([], 'Create new menu import', 'mm-import-type-add-form');

    $this->drupalGet('/admin/config/development/menu-migration/import-types');
    // Verify that the newly created import type is displayed as expected in the
    // listing.
    $this->assertEntityTableColumns([
      'upload_main_nav',
      'Main navigation upload',
      'main',
      'YAML',
      'File Upload',
      'N/A',
    ], 'import');
  }

}
