<?php

declare(strict_types=1);

namespace Drupal\Tests\menu_migration\Functional;

use Drupal\Tests\menu_migration\Traits\TestsHelperTrait;
use Drupal\user\UserInterface;
use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Tests the UI drag and drop for reordering configuration entities.
 *
 * @group menu_migration
 */
class DragAndDropTest extends WebDriverTestBase {
  use TestsHelperTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['menu_migration'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * The admin user that we will work with.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $exportTypes = [
      ['destination' => 'download', 'menus' => 'main'],
      ['destination' => 'codebase', 'menus' => ['main']],
      ['destination' => 'download', 'menus' => 'footer'],
      ['destination' => 'codebase', 'menus' => ['main', 'footer']],
    ];
    foreach ($exportTypes as $delta => $data) {
      $weight = $delta + 1;
      $exportEntity = $this->exportTypeStorage()->create([
        'label' => "Export Type $weight",
        'name' => "export_type_$weight",
        'destination' => $data['destination'],
        'destination_config' => [
          'format' => $delta % 2 == 0 ? 'yaml' : 'json',
          'menus' => $data['menus'],
        ],
        'weight' => $weight,
      ]);
      $importEntity = $this->importTypeStorage()->create([
        'label' => "Import Type $weight",
        'name' => "import_type_$weight",
        'source' => $data['destination'] == 'download' ? 'file_upload' : $data['destination'],
        'source_config' => [
          'format' => $delta % 2 == 0 ? 'yaml' : 'json',
          'menus' => $data['menus'],
        ],
        'weight' => $weight,
      ]);
      if ($data['destination'] == 'codebase') {
        $exportEntity->set('destination_config.export_path', '../assets/menus');
        $importEntity->set('source_config.import_path', '../assets/menus');
      }
      $exportEntity->save();
      $importEntity->save();
    }
    $this->drupalCreateUser();
    $this->adminUser = $this->drupalCreateUser(['administer menu migration']);
  }

  /**
   * Tests the drag and drop functionality on Import and Export listing pages.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Behat\Mink\Exception\ResponseTextException
   */
  public function testDragAndDrop(): void {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/menu-migration/export-types');
    $this->assertDragAndDrop('export');
    $this->drupalGet('/admin/config/development/menu-migration/import-types');
    $this->assertDragAndDrop('import');
  }

  /**
   * Tests that the copy button does not submit the drag and drop form.
   *
   * @see https://www.drupal.org/project/menu_migration/issues/3526804
   *
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   */
  public function testCommandCopyButtonConflict(): void {
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/admin/config/development/menu-migration/export-types');

    $page = $this->getSession()->getPage();
    $this->assertSession()->pageTextNotContains('The listing order has been updated.');
    $page->pressButton('mm-copy-export_type_2');
    $this->assertSession()->waitForText('The listing order has been updated.', 3000);
    $this->assertSession()->pageTextNotContains('The listing order has been updated.');
  }

  /**
   * Performs assertions for Import and Export listings drag and drop feature.
   *
   * @param string $type
   *   A string containing either "import" or "export" that will be used to
   *   check the behavior of the drag and drop listings for Export Type and
   *   Import Type entities.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Behat\Mink\Exception\ResponseTextException
   */
  protected function assertDragAndDrop(string $type): void {
    $this->assertSession()->elementsCount('css', '.tabledrag-handle', 4);

    $session = $this->getSession();
    $page = $session->getPage();
    // Assert initial rows order.
    $labelsXpath = '//*[@id="edit-' . $type . '-types"]/tbody/tr/td[2]';
    $labelPrefix = ucfirst($type);
    $rowLabels = $page->findAll('xpath', $labelsXpath);
    foreach ($rowLabels as $delta => $rowLabel) {
      $expectedLabel = "$labelPrefix Type " . ($delta + 1);
      $this->assertEquals($expectedLabel, $rowLabel->getText());
    }
    // Drag the top row to last row.
    $rows = $page->findAll('css', 'a.tabledrag-handle');
    $rows[0]->dragTo($rows[3]);
    $this->assertSession()->waitForText('You have unsaved changes.');
    // Verify new order.
    $expectedOrder = [2, 3, 4, 1];
    $rowLabels = $page->findAll('xpath', $labelsXpath);
    foreach ($rowLabels as $delta => $rowLabel) {
      $this->assertEquals("$labelPrefix Type $expectedOrder[$delta]", $rowLabel->getText());
    }

    // Test show row weights.
    $toggle = $page->findButton('Show row weights');
    $this->assertNotEmpty($toggle);
    $toggle->click();

    $weightFields = [];
    foreach ($expectedOrder as $delta => $value) {
      $weightFields[$delta] = $page->findField("{$type}_types[{$type}_type_$value][weight]");
      $this->assertTrue($weightFields[$delta]->isVisible());
    }
    // Check that weights were changed.
    $this->assertLessThan($weightFields[1]->getValue(), $weightFields[0]->getValue());
    $this->assertLessThan($weightFields[2]->getValue(), $weightFields[1]->getValue());
    $this->assertLessThan($weightFields[3]->getValue(), $weightFields[2]->getValue());

    // Toggle back the show row weights.
    $toggle = $page->findButton('Hide row weights');
    $this->assertNotEmpty($toggle);
    $toggle->click();

    $this->submitForm([], 'Save');
    $this->assertSession()->waitForText('The listing order has been updated.');
    $this->assertSession()->pageTextContains('The listing order has been updated.');

    // Check the order after save.
    foreach ($expectedOrder as $delta => $value) {
      $weightFields[$delta] = $page->findField("{$type}_types[{$type}_type_$value][weight]");
      $this->assertFalse($weightFields[$delta]->isVisible());
    }
  }

}
