<?php

declare(strict_types=1);

namespace Drupal\Tests\menu_migration\Kernel;

use Drupal\Core\Access\AccessManagerInterface;
use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\menu_migration\Entity\ExportTypeInterface;
use Drupal\menu_migration\Entity\ImportTypeInterface;
use Drupal\Tests\menu_migration\Traits\TestsHelperTrait;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\user\UserInterface;

/**
 * Tests the access to entities and pages.
 *
 * @group menu_migration
 */
class AccessTest extends KernelTestBase {
  use UserCreationTrait;
  use TestsHelperTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'menu_migration',
  ];

  /**
   * The export type entity that will be used for testing.
   *
   * @var \Drupal\menu_migration\Entity\ExportTypeInterface
   */
  protected ExportTypeInterface $exportType;

  /**
   * The import type entity that will be used for testing.
   *
   * @var \Drupal\menu_migration\Entity\ImportTypeInterface
   */
  protected ImportTypeInterface $importType;

  /**
   * The access manager service.
   *
   * @var \Drupal\Core\Access\AccessManagerInterface
   */
  protected AccessManagerInterface $accessManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installEntitySchema('user');
    $this->installEntitySchema('mm_export_type');
    $this->accessManager = $this->container->get('access_manager');

    $this->exportType = $this->exportTypeStorage()->create([
      'label' => 'Test Codebase Export',
      'name' => 'test_codebase_export',
      'destination' => 'codebase',
      'destination_config' => [
        'format' => 'yaml',
        'menus' => [],
        'export_path' => '../assets/menus',
      ],
    ]);
    $this->exportType->save();
    $this->importType = $this->importTypeStorage()->create([
      'label' => 'Test Codebase Import',
      'name' => 'test_codebase_import',
      'source' => 'codebase',
      'source_config' => [
        'format' => 'yaml',
        'menus' => [],
        'import_path' => '../assets/menus',
      ],
    ]);
    $this->importType->save();
    // Create a user so the tested users do not have the ID 1.
    $this->createUser();
  }

  /**
   * Tests the "administer menu migration" permission and accesses.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testBypassAccess(): void {
    $user = $this->createUser(['administer menu migration']);

    $this->checkEntityOperationsAccess($this->exportType, $user, TRUE, TRUE, TRUE, TRUE, FALSE);
    $this->checkEntityLinkTemplatesAccess($this->exportType, $user, TRUE, TRUE, TRUE, TRUE, TRUE);

    $this->checkEntityOperationsAccess($this->importType, $user, TRUE, TRUE, TRUE, FALSE, TRUE);
    $this->checkEntityLinkTemplatesAccess($this->importType, $user, TRUE, TRUE, TRUE, TRUE, TRUE);

    $this->checkCustomRoutesAccess(TRUE, $user);
  }

  /**
   * Tests the export types administration access.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testAdministerExportTypesAccess(): void {
    $user = $this->createUser(['administer menu migration export types']);

    $this->checkEntityOperationsAccess($this->exportType, $user, TRUE, TRUE, TRUE, TRUE, FALSE);
    $this->checkEntityLinkTemplatesAccess($this->exportType, $user, TRUE, TRUE, TRUE, TRUE, TRUE);

    $this->checkEntityOperationsAccess($this->importType, $user, FALSE, FALSE, FALSE, FALSE, FALSE);
    $this->checkEntityLinkTemplatesAccess($this->importType, $user, FALSE, FALSE, FALSE, FALSE, FALSE);

    $this->checkCustomRoutesAccess(FALSE, $user);
  }

  /**
   * Tests the import types administration access.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testAdministerImportTypesAccess(): void {
    $user = $this->createUser(['administer menu migration import types']);

    $this->checkEntityOperationsAccess($this->exportType, $user, FALSE, FALSE, FALSE, FALSE, FALSE);
    $this->checkEntityLinkTemplatesAccess($this->exportType, $user, FALSE, FALSE, FALSE, FALSE, FALSE);

    $this->checkEntityOperationsAccess($this->importType, $user, TRUE, TRUE, TRUE, FALSE, TRUE);
    $this->checkEntityLinkTemplatesAccess($this->importType, $user, TRUE, TRUE, TRUE, TRUE, TRUE);

    $this->checkCustomRoutesAccess(FALSE, $user);
  }

  /**
   * Tests the "perform exports" access.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testPerformExportsAccess(): void {
    $user = $this->createUser(['perform export on menu migrations']);

    $this->checkEntityOperationsAccess($this->exportType, $user, FALSE, FALSE, FALSE, TRUE, FALSE);
    $this->checkEntityLinkTemplatesAccess($this->exportType, $user, FALSE, FALSE, FALSE, TRUE, TRUE);

    $this->checkEntityOperationsAccess($this->importType, $user, FALSE, FALSE, FALSE, FALSE, FALSE);
    $this->checkEntityLinkTemplatesAccess($this->importType, $user, FALSE, FALSE, FALSE, FALSE, FALSE);

    $this->checkCustomRoutesAccess(FALSE, $user);
  }

  /**
   * Tests the "perform imports" access.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testPerformImportsAccess(): void {
    $user = $this->createUser(['perform import on menu migrations']);

    $this->checkEntityOperationsAccess($this->exportType, $user, FALSE, FALSE, FALSE, FALSE, FALSE);
    $this->checkEntityLinkTemplatesAccess($this->exportType, $user, FALSE, FALSE, FALSE, FALSE, FALSE);

    $this->checkEntityOperationsAccess($this->importType, $user, FALSE, FALSE, FALSE, FALSE, TRUE);
    $this->checkEntityLinkTemplatesAccess($this->importType, $user, FALSE, FALSE, FALSE, TRUE, TRUE);

    $this->checkCustomRoutesAccess(FALSE, $user);
  }

  /**
   * Checks if a user has specific operation access to export/import entities.
   *
   * @param \Drupal\Core\Config\Entity\ConfigEntityInterface $entity
   *   The ExportType or ImportType entity to check against.
   * @param \Drupal\user\UserInterface $user
   *   The user entity.
   * @param bool $opCreate
   *   The expected value of the entity create access.
   * @param bool $opUpdate
   *   The expected value of the entity update access.
   * @param bool $opDelete
   *   The expected value of the entity delete access.
   * @param bool $opExport
   *   The expected value of the entity export access.
   * @param bool $opImport
   *   The expected value of the entity import access.
   */
  protected function checkEntityOperationsAccess(ConfigEntityInterface $entity, UserInterface $user, bool $opCreate, bool $opUpdate, bool $opDelete, bool $opExport, bool $opImport): void {
    $this->checkEntityAccess($entity, [
      'create' => $opCreate,
      'update' => $opUpdate,
      'delete' => $opDelete,
      'export' => $opExport,
      'import' => $opImport,
    ], $user);
  }

  /**
   * Checks if a user has the given entity operations accesses.
   *
   * @param \Drupal\Core\Config\Entity\ConfigEntityInterface $entity
   *   The ExportType or ImportType entity to check against.
   * @param array $operations
   *   An array containing the entity operation and its expected boolean assert
   *   result.
   * @param \Drupal\user\UserInterface $user
   *   The user entity.
   */
  protected function checkEntityAccess(ConfigEntityInterface $entity, array $operations, UserInterface $user): void {
    foreach ($operations as $operation => $expectedResult) {
      $message = sprintf('The %s entity %s operation access is %s for user.', $entity->getEntityTypeId(), $operation, $expectedResult ? 'accessible' : 'restricted');
      $this->assertEquals($expectedResult, $entity->access($operation, $user), $message);
    }
  }

  /**
   * Checks the access of a user to the given entity's link templates.
   *
   * @param \Drupal\Core\Config\Entity\ConfigEntityInterface $entity
   *   The ExportType or ImportType entity to check against.
   * @param \Drupal\user\UserInterface $user
   *   The user entity.
   * @param bool $addForm
   *   The expected value of the entity add-form link template access.
   * @param bool $editForm
   *   The expected value of the entity edit-form link template access.
   * @param bool $deleteForm
   *   The expected value of the entity delete-form link template access.
   * @param bool $importExportForm
   *   The expected value of the entity import-form or export-form link template
   *   access.
   * @param bool $collection
   *   The expected value of the entity collection link template access.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  protected function checkEntityLinkTemplatesAccess(ConfigEntityInterface $entity, UserInterface $user, bool $addForm, bool $editForm, bool $deleteForm, bool $importExportForm, bool $collection): void {
    $importExportKey = $entity instanceof ExportTypeInterface ? 'export-form' : 'import-form';
    $this->checkEntityUrlAccess($entity, [
      'add-form' => $addForm,
      'edit-form' => $editForm,
      'delete-form' => $deleteForm,
      $importExportKey => $importExportForm,
      'collection' => $collection,
    ], $user);
  }

  /**
   * Checks the user access against an entity's URLs.
   *
   * @param \Drupal\Core\Config\Entity\ConfigEntityInterface $entity
   *   The ExportType or ImportType entity to check against.
   * @param array $linkTemplates
   *   An array containing the entity link template and its expected boolean
   *   assert result.
   * @param \Drupal\user\UserInterface $user
   *   The user entity.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  protected function checkEntityUrlAccess(ConfigEntityInterface $entity, array $linkTemplates, UserInterface $user): void {
    foreach ($linkTemplates as $linkTemplate => $expectedAccess) {
      $message = sprintf('The %s entity %s url is %s for user.', $entity->getEntityTypeId(), $linkTemplate, $expectedAccess ? 'accessible' : 'not accessible');
      $this->assertEquals($expectedAccess, $entity->toUrl($linkTemplate)->access($user), $message);
    }
  }

  /**
   * Checks the user access against custom module routes.
   *
   * @param bool $expectedResult
   *   A boolean indicating the expected access result.
   * @param \Drupal\user\UserInterface $user
   *   The user entity to check against.
   */
  protected function checkCustomRoutesAccess(bool $expectedResult, UserInterface $user): void {
    $accessResult = $this->accessManager->checkNamedRoute('menu_migration.quick_action_settings', [], $user);
    $message = sprintf('The user %s access to menu_migration.quick_action_settings route.', $expectedResult ? 'has' : 'does not have');
    $this->assertEquals($expectedResult, $accessResult, $message);
  }

}
