<?php

declare(strict_types=1);

namespace Drupal\Tests\menu_migration\Kernel;

use Drupal\Core\Menu\MenuTreeParameters;

/**
 * Tests the import and export of menu items.
 *
 * @group menu_migration
 */
class MenuImportExportTest extends MenuMigrationKernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'menu_link_content',
    'link',
    'user',
    'menu_migration',
  ];

  /**
   * Tests that the newly created menu exists.
   */
  public function testMenuExistence(): void {
    $dbMenu = $this->menuStorage()->load($this->menu->id());
    $this->assertEquals($this->menu->id(), $dbMenu->id(), 'The newly created menu exists.');
  }

  /**
   * Tests the basic export and import of menu items.
   */
  public function testBasicExportImport(): void {
    $link1 = $this->createMenuLink([
      'title' => 'Link Title',
      'description' => 'Link Description',
      'link' => [
        'uri' => 'internal:/',
        'options' => ['attributes' => ['class' => ['some-class']]],
      ],
      'menu_name' => $this->menu->id(),
      'weight' => 13,
      'enabled' => 1,
      'expanded' => 1,
    ]);
    $link1->save();
    $this->assertMenuLinksCount(1);

    $this->exportMenus();
    $this->importMenus();

    $tree = $this->loadMenuTree();
    $this->assertCount(1, $tree, 'The menu link tree contains 1 element.');
    $dbLink = reset($tree);
    $this->assertMenuLinkElementEquals($link1, $dbLink);
    $this->assertNotEmpty($dbLink->link->getOptions(), 'The menu link options are populated.');
    $this->assertEquals($link1->getPluginDefinition()['options'], $dbLink->link->getOptions(), 'The menu link options are the same.');
  }

  /**
   * Tests the export and import of hierarchical menu items.
   */
  public function testHierarchy(): void {
    $hierarchy = $this->createSimpleMenuHierarchy();
    $this->assertMenuLinksCount(3);

    $this->exportMenus();
    $this->importMenus();
    $this->assertMenuLinksCount(3);

    $menuTree = $this->menuLinkTree()->load($this->menu->id(), new MenuTreeParameters());
    $dbParent = reset($menuTree);
    $this->assertMenuLinkElementEquals($hierarchy['parent'], $dbParent);

    $dbChild = reset($dbParent->subtree);
    $this->assertMenuLinkElementEquals($hierarchy['child'], $dbChild);

    $dbSubChild = reset($dbChild->subtree);
    $this->assertMenuLinkElementEquals($hierarchy['sub_child'], $dbSubChild);
  }

  /**
   * Tests the export of subtree containing only disabled items.
   *
   * @see https://www.drupal.org/project/menu_migration/issues/3476931
   */
  public function testDisabledSubtree(): void {
    $parent = $this->createMenuLink([
      'title' => 'Parent',
      'link' => ['uri' => 'internal:/#parent'],
      'menu_name' => $this->menu->id(),
    ]);
    $parent->save();
    // Add 3 disabled children.
    foreach (range(1, 3) as $delta) {
      $this->createMenuLink([
        'title' => 'Child ' . $delta,
        'link' => ['uri' => 'internal:/#child' . $delta],
        'menu_name' => $this->menu->id(),
        'parent' => 'menu_link_content:' . $parent->uuid(),
        'enabled' => FALSE,
      ])->save();
    }

    $this->exportMenus();
    $this->importMenus();

    $menuTree = $this->loadMenuTree();
    $this->assertCount(1, $menuTree);
    $dbParent = reset($menuTree);
    $this->assertCount(3, $dbParent->subtree);
  }

}
