<?php

declare(strict_types=1);

namespace Drupal\Tests\menu_migration\Kernel;

use Drupal\Core\Menu\MenuLinkTreeElement;
use Drupal\Core\Menu\MenuTreeParameters;
use Drupal\KernelTests\KernelTestBase;
use Drupal\menu_link_content\MenuLinkContentInterface;
use Drupal\system\MenuInterface;
use Drupal\Tests\menu_migration\Traits\TestsHelperTrait;

/**
 * Provides a base Kernel test class for menu_migration tests.
 *
 * @group menu_migration
 */
class MenuMigrationKernelTestBase extends KernelTestBase {
  use TestsHelperTrait;

  /**
   * The export format that will be used throughout the tests.
   *
   * @var string
   */
  protected string $exportFormat = 'yaml';

  /**
   * The test menu entity that we will work with.
   *
   * @var \Drupal\system\MenuInterface
   */
  protected MenuInterface $menu;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('menu');
    $this->installEntitySchema('menu_link_content');

    $this->menu = $this->menuStorage()->create([
      'id' => 'menu-test',
      'label' => 'Test menu',
      'description' => 'Test menu description',
    ]);
    $this->menu->save();
  }

  /**
   * Creates a menu link item.
   *
   * @param array $properties
   *   An array containing the properties for the menu link item.
   *
   * @return \Drupal\menu_link_content\MenuLinkContentInterface
   *   The menu link content entity.
   */
  protected function createMenuLink(array $properties): MenuLinkContentInterface {
    /** @var \Drupal\menu_link_content\MenuLinkContentInterface $link */
    $link = $this->menuLinkContentStorage()->create($properties);
    return $link;
  }

  /**
   * Creates a simple menu items hierarchy.
   *
   * @return \Drupal\menu_link_content\MenuLinkContentInterface[]
   *   An array containing the hierarchy as parent, child and sub_child.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function createSimpleMenuHierarchy(): array {
    $parent = $this->createMenuLink([
      'title' => 'Parent 1',
      'description' => 'Parent Menu Item Description',
      'link' => ['uri' => 'internal:/#parent1'],
      'menu_name' => $this->menu->id(),
    ]);
    $parent->save();
    $child = $this->createMenuLink([
      'title' => 'Child 1',
      'link' => ['uri' => 'internal:/#child1'],
      'description' => 'Child Menu Item Description',
      'menu_name' => $this->menu->id(),
      'parent' => 'menu_link_content:' . $parent->uuid(),
    ]);
    $child->save();
    $subChild = $this->createMenuLink([
      'title' => 'Sub Child 1',
      'description' => 'Sub Child Menu Item Description',
      'link' => ['uri' => 'internal:/#subchild1'],
      'menu_name' => $this->menu->id(),
      'parent' => 'menu_link_content:' . $child->uuid(),
    ]);
    $subChild->save();
    return [
      'parent' => $parent,
      'child' => $child,
      'sub_child' => $subChild,
    ];
  }

  /**
   * Loads the menu tree for the given menu ID.
   *
   * @param string|null $menuId
   *   The menu ID if provided, otherwise the default will be used.
   *
   * @return \Drupal\Core\Menu\MenuLinkTreeElement[]
   *   An array containing the menu link tree.
   */
  protected function loadMenuTree(?string $menuId = NULL): array {
    $menuId = $menuId ?? $this->menu->id();
    return $this->menuLinkTree()->load($menuId, new MenuTreeParameters());
  }

  /**
   * Loads the menu link content object from the given menu link tree element.
   *
   * @param \Drupal\Core\Menu\MenuLinkTreeElement $linkTreeElement
   *   The menu link tree element object.
   *
   * @return \Drupal\menu_link_content\MenuLinkContentInterface
   *   Returns the menu link content entity.
   */
  protected function loadMenuLinkFromTreeElement(MenuLinkTreeElement $linkTreeElement): MenuLinkContentInterface {
    $itemId = $linkTreeElement->link->getMetaData()['entity_id'];
    /** @var \Drupal\menu_link_content\MenuLinkContentInterface $menuLink */
    $menuLink = $this->menuLinkContentStorage()->load($itemId);
    return $menuLink;
  }

  /**
   * Exports the given menu IDs to Codebase.
   *
   * @param array $menus
   *   An array containing the menu IDs to be imported. If empty, the default
   *   menu ID will be used.
   */
  protected function exportMenus(array $menus = []): void {
    $menus = empty($menus) ? [$this->menu->id()] : $menus;
    $exportType = $this->exportTypeStorage()->create([
      'label' => 'Test Codebase Export',
      'name' => 'test_codebase_export',
      'destination' => 'codebase',
      'destination_config' => [
        'format' => $this->exportFormat,
        'menus' => $menus,
        'export_path' => '../assets/menus',
      ],
    ]);
    $exportType->export();

    foreach ($menus as $menu) {
      $this->assertFileExists(DRUPAL_ROOT . "/../assets/menus/$menu.$this->exportFormat", 'The generated export file exists.');
    }
  }

  /**
   * Imports the given menus using the Codebase import plugin.
   *
   * @param array $menus
   *   An array containing the menu IDs to be imported. If empty, the default
   *   menu ID will be used.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function importMenus(array $menus = []): void {
    $menus = empty($menus) ? [$this->menu->id()] : $menus;
    $importType = $this->importTypeStorage()->create([
      'label' => 'Test Codebase Import',
      'name' => 'test_codebase_import',
      'source' => 'codebase',
      'source_config' => [
        'format' => $this->exportFormat,
        'menus' => $menus,
        'import_path' => '../assets/menus',
      ],
    ]);
    $importType->import();

    foreach ($menus as $menu) {
      // Cleanup import files.
      unlink(DRUPAL_ROOT . "/../assets/menus/$menu.$this->exportFormat");
      $this->assertFileDoesNotExist(DRUPAL_ROOT . "/../assets/menus/$menu.$this->exportFormat", 'The deleted source file does not exist.');
    }
  }

  /**
   * Compares the basic properties of two menu item content links.
   *
   * The expected menu item for comparison is supposed to be an imported menu
   * item, which means that the pluginID should always be different from the
   * original menu item.
   *
   * @param \Drupal\menu_link_content\MenuLinkContentInterface $expectedLink
   *   The original menu content link item object.
   * @param \Drupal\menu_link_content\MenuLinkContentInterface $actualLink
   *   The imported menu content link item object.
   * @param array $optionalFields
   *   An array containing the machine names of the optional additional fields
   *   to check.
   */
  protected function assertBasicMenuLinkContentEquals(MenuLinkContentInterface $expectedLink, MenuLinkContentInterface $actualLink, array $optionalFields = []): void {
    $this->assertNotEquals($expectedLink->getPluginId(), $actualLink->getPluginId(), 'The menu link item has the expected Plugin ID.');
    $this->assertEquals($expectedLink->getTitle(), $actualLink->getTitle(), 'The menu link item has the expected title.');
    $this->assertEquals($expectedLink->getDescription(), $actualLink->getDescription(), 'The menu link item has the expected description.');
    $this->assertEquals($expectedLink->getUrlObject()->toString(), $actualLink->getUrlObject()->toString(), 'The menu link item has the expected url.');
    $this->assertEquals($expectedLink->isExpanded(), $actualLink->isExpanded(), 'The menu link item has the expected expanded value.');
    $this->assertEquals($expectedLink->isEnabled(), $actualLink->isEnabled(), 'The menu link item has the expected enabled value.');
    $this->assertEquals($expectedLink->getWeight(), $actualLink->getWeight(), 'The menu link item has the expected weight.');
    foreach ($optionalFields as $fieldName) {
      $this->assertEquals($expectedLink->get($fieldName)->getValue(), $actualLink->get($fieldName)->getValue(), 'The menu link item has the expected optional field value.');
    }
  }

  /**
   * Compares the basic properties of a menu item content link and tree element.
   *
   * @param \Drupal\menu_link_content\MenuLinkContentInterface $expectedLink
   *   The original menu content link item object.
   * @param \Drupal\Core\Menu\MenuLinkTreeElement $menuLinkTreeElement
   *   The menu link tree element for the menu link content item.
   * @param array $optionalFields
   *   An optional array containing any additional extra field names.
   */
  protected function assertMenuLinkElementEquals(MenuLinkContentInterface $expectedLink, MenuLinkTreeElement $menuLinkTreeElement, array $optionalFields = []): void {
    $actualLink = $this->loadMenuLinkFromTreeElement($menuLinkTreeElement);
    $this->assertBasicMenuLinkContentEquals($expectedLink, $actualLink, $optionalFields);
  }

  /**
   * Asserts the number of available menu items in a menu.
   *
   * @param int $expectedCount
   *   The expected number of menu items.
   * @param string|null $menuId
   *   The menu ID if given, otherwise the default test menu ID will be used.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function assertMenuLinksCount(int $expectedCount, ?string $menuId = NULL): void {
    $menuId = $menuId ?? $this->menu->id();
    $links = $this->menuLinkContentStorage()->loadByProperties(['menu_name' => $menuId]);
    $this->assertCount($expectedCount, $links, 'The given menu has the expected number of menu link items.');
  }

}
