<?php

namespace Drupal\Tests\menu_migration\Traits;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Drupal\menu_link_content\MenuLinkContentStorageInterface;

/**
 * Provides a helper trait for menu_migration tests.
 *
 * @internal
 */
trait TestsHelperTrait {

  /**
   * Places the default blocks needed for functional testing.
   *
   * @throws \Exception
   */
  protected function placeDefaultBlocks(): void {
    if (method_exists($this, 'drupalPlaceBlock')) {
      // Place default blocks.
      $this->drupalPlaceBlock('system_main_block', ['region' => 'content']);
      $this->drupalPlaceBlock('page_title_block', ['region' => 'highlighted', 'weight' => 0]);
      $this->drupalPlaceBlock('local_tasks_block', ['region' => 'highlighted', 'weight' => 1]);
      $this->drupalPlaceBlock('local_actions_block', ['region' => 'highlighted', 'weight' => 2]);
    }
    else {
      throw new \Exception('The "drupalPlaceBlock" method is not available in this context.');
    }
  }

  /**
   * Checks the expected values in the export/import entity listing tables.
   *
   * @param array $tableColumns
   *   An array containing the values of a row in the table.
   * @param string $type
   *   A string indicating if the listing is of type "export" or "import".
   */
  protected function assertEntityTableColumns(array $tableColumns, string $type): void {
    if (method_exists($this, 'assertSession')) {
      foreach ($tableColumns as $delta => $columnText) {
        $selector = '//*[@id="edit-' . $type . '-types"]/tbody/tr/td[' . $delta + 1 . ']';
        $this->assertSession()->elementTextEquals('xpath', $selector, $columnText);
      }
    }
    else {
      throw new \Exception('The "assertSession" method is not available in this context.');
    }
  }

  /**
   * Gets the entity type manager service.
   *
   * @return \Drupal\Core\Entity\EntityTypeManagerInterface
   *   The entity type manager service.
   */
  protected function entityTypeManager(): EntityTypeManagerInterface {
    return $this->container->get('entity_type.manager');
  }

  /**
   * Gets the Export Type entity storage.
   *
   * @return \Drupal\Core\Entity\EntityStorageInterface
   *   The Export Type entity storage.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function exportTypeStorage(): EntityStorageInterface {
    return $this->entityTypeManager()->getStorage('mm_export_type');
  }

  /**
   * Gets the Import Type entity storage.
   *
   * @return \Drupal\Core\Entity\EntityStorageInterface
   *   The Import Type entity storage.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function importTypeStorage(): EntityStorageInterface {
    return $this->entityTypeManager()->getStorage('mm_import_type');
  }

  /**
   * Gets the Menu Link Content entity storage.
   *
   * @return \Drupal\menu_link_content\MenuLinkContentStorageInterface
   *   The Menu Link Content entity storage.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function menuLinkContentStorage(): MenuLinkContentStorageInterface {
    return $this->entityTypeManager()->getStorage('menu_link_content');
  }

  /**
   * Gets the Menu entity storage.
   *
   * @return \Drupal\Core\Entity\EntityStorageInterface
   *   The Menu entity storage.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function menuStorage(): EntityStorageInterface {
    return $this->entityTypeManager()->getStorage('menu');
  }

  /**
   * Gets the menu link tree service.
   *
   * @return \Drupal\Core\Menu\MenuLinkTreeInterface
   *   The menu link tree service.
   *
   * @throws \Exception
   */
  protected function menuLinkTree(): MenuLinkTreeInterface {
    return $this->container->get('menu.link_tree');
  }

}
